<?php

use Illuminate\Support\Arr;
use Illuminate\Support\Str;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\File;


if ( !function_exists('calc_increase_percent')) {
    /**
     * Calcul le pourcentage d'accroissement entre 2 valeurs
     * @param  int|float $old_value
     * @param  int|float $new_value
     * @return float            La valeur d'accroissement
     */
    function calc_increase_percent($old_value, $new_value)
    {
        if ($new_value == 0) {
            return 0;
        } else if ($old_value == 0) {
            return 100;
        }

        // Calcul du rapport entre la nouvelle valeur et l'ancienne
        $ratio = ($new_value - $old_value) / $old_value;

        // Convertion en pourcentage
        $percent = round($ratio * 100);

        return $percent;
    }
}

if (! function_exists('mixte_get')) {
    /**
     * Récupération avancée de contenu depuis une source Mixte, en utilisant la notation «.»
     * Possibilité d'appeler des méthodes en leur passant des arguments
     * Ex : mixte_get($collection, "where('name', 'toto').first()");
     *
     * @param  mixed   $target
     * @param  string|array|int  $key
     * @param  mixed   $default
     * @return mixed
     */
    function mixte_get($target, $key, $default = null)
    {
        if (is_null($key)) {
            return $target;
        }

        // Convertion des « -> » en « . » (Compatibilité)
        $key = str_replace('->', '.', $key);

        $key = is_array($key) ? $key : explode('.', $key);

        while (! is_null($segment = array_shift($key))) {
            if ($segment === '*') {
                if ($target instanceof Collection) {
                    $target = $target->all();
                } elseif (! is_array($target)) {
                    return value($default);
                }

                $result = [];

                foreach ($target as $item) {
                    $result[] = data_get($item, $key);
                }

                return in_array('*', $key) ? Arr::collapse($result) : $result;
            }

            $params = '';
            $args = null;
            if (Str::contains($segment, ['(', ')'])) {

                if (preg_match('/\((.*)\)/', $segment, $finded)) {
                    $segment = str_replace($finded[0], '', $segment);
                    $params = $finded[1];
                    $args = str_args_to_array($params);
                }
            }

            if (Arr::accessible($target) && Arr::exists($target, $segment)) {
                $target = $target[$segment];
            } elseif (is_object($target) && isset($target->{$segment})) {
                $target = $target->{$segment};
            } elseif (is_object($target) && method_exists($target, $segment)) {

                if (!is_null($args)) {
                    // Appel de la methode avec des arguments
                    $target = call_user_func_array([$target, $segment], $args);

                } else {
                    // Appel de la methode sans argument
                    $target = $target->{$segment}();
                }
            } else {
                return value($default);
            }
        }

        return $target;
    }
}

if ( !function_exists('mixte_get_multi')) {
    /**
     * Récupère un tableau de données depuis une cible à partir d'un tableau de clés
     * @param  Mixed $target    La donnée source
     * @param  Array $keys      Tableau des clés à rechercher
     * @param  Mixed $default   Valeur par défaut
     * @return Array            Tableau des données récoltées
     */
    function mixte_get_multi($target, array $keys, $default = null)
    {
        $datas = array();
        foreach ($keys as $key) {
            $datas[] = mixte_get($target, $key, $default);
        }
        return $datas;
    }
}

if ( !function_exists('public_folder_to_urls')) {
    /**
     * Récupère une collection d'url de fichiers depuis le chemin relatif d'un dossier public
     *
     * @param string $public_path
     * @return Collection
     */
    function public_folder_to_urls($public_path)
    {
        $files_path = collect();
        if (!empty($public_path)) {

            $path = public_path($public_path);

            $files = File::files($path);
            $base_path = public_path();

            foreach ($files as $file) {
                $url = str_replace($base_path, url('/'), $file->getPathName());
                $files_path->push($url);
            }
        }

        return $files_path;
    }
}
